
//some global vars 
PImage b = loadImage("images/internal.png");
//PFont font = loadFont("Arial");			//set up the font
//textFont(font,24);

float  zOffset = 1.03;    //adjust for earth's gravity at your location (should be around 1)

//current index
int Idx = 0;
int currValIdx = 0;
//weight and height of the whole thing
int base_size = 360;
int w = base_size, h = w;
//width and h of the image that is to be used as back of the canvas
int img_wid = 242;
int img_hei = 242;

//fps to draw at
int fps;
int fps2draw_ratio = 1;
int max_fps = 60;

//array for timestamps used for calculating fps over last x frames
float fps_stamps = new Array();
int fps_numint = 10;

//storage for data points
int dataPoints = round(w * 1.5);
float[][] Histo = new float[dataPoints][3];

int gridLength = w;


float   Y_scale_mult = 1;
float   scaleX = 1;
float   scaleY = h/2 * Y_scale_mult; 
int     deltaX = round((float)gridLength/dataPoints);

float curve_vals;
color line_cols = [ color(255,0,0), color(0,255,0), color(0,0,255)];
var label_char = ["x", "y", "z"];
int label_offsets;
int char_off = 24;
int hor_off = 12;


//now for the options
boolean colors;	
boolean labels;
boolean legend;
boolean legend_vals;
float rel_size = 1;
int line_style;
//the local looping var, is set by page to signal that other canvas is active
boolean local_looping = true;

//---------------------- this is where the actual code starts ---------


void setup(){

  noLoop();
  	reset_vars();
  	setupLineGraph();
  	if (fps>0)
		frameRate(fps);
	else
		frameRate(1);
  loop();
  size(w, h);
  smooth();
	
}

void clear_array(){
	for (int i=0; i<dataPoints; i++){
	  Histo[i][0] = Number.MIN_VALUE;
	  Histo[i][1] = Number.MIN_VALUE;
	  Histo[i][2] = Number.MIN_VALUE;
	}
}

clear_array();

void pause_local(){ local_looping = false; }
void continue_local(){ local_looping = true; }

void reset_vars(){
	 var size_changed = false;
	 colors 		= window.Processing.data.colors;
	 labels 		= window.Processing.data.labels;
	 legend 		= window.Processing.data.legend;
	 legend_vals	= window.Processing.data.vals;
	 line_style 	= window.Processing.data.line_style;
	 flips 			= window.Processing.data.flips;
	 fps			= round(window.Processing.data.fps * max_fps);
	 zOffset		= window.Processing.data.delta_z;

	 if (rel_size != window.Processing.data.rel_size){

			//reset the  vars that depend on size
			int new_size = round(window.Processing.data.rel_size * base_size);
			w = new_size; h = w;
			dataPoints = round(w * 1.5);
			Histo = new float[dataPoints][3];
			gridLength = w;
			scaleY = h/2 * Y_scale_mult; 
			deltaX = round((float)gridLength/dataPoints);
			clear_array();
			rel_size		= window.Processing.data.rel_size;
	}
	
	 local_looping = true;
}

var 	flips;// 		= window.Processing.data.flips;


void draw(){
	if (	!local_looping || 
			(	
				window.Processing.data.pause_hidden &&
				!window.Processing.data.draw_looping
			)
		) 
		return;
	
    getReading();

  //see if we need to redraw
  if ((fps2draw_ratio>1) && (frameCount % fps2draw_ratio) == 0)
    return;

	background(new color(255,255,255));

  //draw the bg
  for (int i = 0; i < (w/img_wid + 1) ; i++){
    for (int j = 0; j < (h/img_hei+1) ; j++){
      image(b, (i*img_wid), (j*img_hei));
    }
  }
    
	//draw graphs
    drawLineGraph();
  
    if (legend_vals)
      update_vals();

	
	if (window.Processing.data.show_fps){
		//calc fps
		if (fps_stamps.push(Date.now()) > fps_numint){
			var last_ts = fps_stamps.shift();
			$("#fps_val").html(((fps_numint / (Date.now() - last_ts))*1000).toFixed(2) + "fps");
		}
	}
}


void getReading(){

  //store new reading into histo array
   Histo[Idx] = getSmsVal();

	//and adjust the offset
   Histo[Idx][2] += zOffset;
  
  currValIdx = Idx;
  //and advance index
  Idx = (Idx+1) % dataPoints;
  
}




// ------------------------------------------------------- the graphs -------- 



void setupLineGraph(){
   
	 curve_vals 	= null;
	 label_offsets	= null;


    float new_curve_vals = [
				[ 
                   [ ( flips[0] ? 0 : w), round(h/4),     ( flips[0] ? 0 : PI) ],
                   [ ( flips[1] ? 0 : w), (2*round(h/4)), ( flips[1] ? 0 : PI) ],
                   [ ( flips[2] ? 0 : w), (3*round(h/4)), ( flips[2] ? 0 : PI) ]
              	],
				[ 
     	           [ w/2,                 ( flips[0] ? h : 0), ( flips[0] ? 3*PI/2 : PI/2) ],
                   [ ( flips[1] ? 0 : w), (round(h/3)),            ( flips[1] ? 0 : PI)        ],
                   [ ( flips[2] ? 0 : w), (2*round(h/3)),          ( flips[2] ? 0 : PI)        ]
               	],
				[ 
			        [ w/2,                   ( flips[0] ? h : 0), ( flips[0] ? 3*PI/2 : PI/2)   ],
                	[ ( flips[1] ? 0 : w),   (round(h/2)),            ( flips[1] ? 0 : PI)          ],
              		[ ( flips[2] ? 0 : w),   ( flips[2] ? h : 0), ( flips[2] ? 7*PI/4 : 3*PI/4) ]
				]	
	];

    int new_offs = [
				[     
                   [ ( flips[0] ? char_off : -char_off), -hor_off ],
                   [ ( flips[1] ? char_off : -char_off), -hor_off ],
                   [ ( flips[2] ? char_off : -char_off), -hor_off ]
                ],
				[     
			       [ hor_off, ( flips[0] ? -char_off : char_off) ],
                   [ ( flips[1] ? char_off : -char_off), -hor_off ],
                   [ ( flips[2] ? char_off : -char_off), -hor_off ]
    			],
				[     
			      [ hor_off, ( flips[0] ? -char_off : char_off) ],
                  [ ( flips[1] ? char_off : -char_off), -hor_off ],
                  [ ( flips[2] ? 2*char_off : -char_off), ( flips[2] ? -char_off : 2*char_off) ]
   				]
];

	curve_vals = new_curve_vals[line_style];
	label_offsets = new_offs[line_style];

}
     
void drawLineGraph(){
       
    //now we just draw each graph seperately by calling drawCurve
     for (int i=0; i < 3; i++){  
       if (colors){
         stroke(line_cols[i]);
         fill(line_cols[i]);
       }else{
	     stroke(0);
         fill(0);
   	   }
       drawCurve((int)curve_vals[i][0], (int)curve_vals[i][1], curve_vals[i][2], flips[i], i);
       
     }
  }
  
  
  //just draw a curve with specified params
  //at point x, y, rotate around rot (rad), flip the vals if neccessary, and which values to draw (0=x,1=y, or 2=z)
  void drawCurve(int x, int y, float rot, boolean flip, int which){
    pushMatrix();
    translate(x,y);
    rotate(rot);
    
    noFill();
    beginShape();
    float xx = 0;
    
    for (int i = Idx-1; i != Idx; i--){
      
      if (i<0) i = dataPoints;
      //check if we are at the end
      if (i == dataPoints || Histo[i][which] == Number.MIN_VALUE) continue;
      
      //get [3] vals using index i
      float v = Histo[i][which];
      
      int flip_mult = 1;
      if (flip) flip_mult *= -1;
      
      int yy = round(v*scaleY) * flip_mult;
      vertex(xx, yy);  
      
      xx += deltaX;
    }  
    
    endShape();
    popMatrix();  
    
  }
  
  void update_vals(){
    for (int i=0; i < 3; i++){  
		var val = Histo[currValIdx][i];
		var label_str = (val>=0 ? '+':'') + val.toFixed(4);
		$("#val_"+i).html(label_str);
	}
 	   
  }